<?php namespace MasterPopups\Includes\ServiceIntegration;

use MaxLopez\HTTPClientWP\IronMan;
use MasterPopups\Includes\Functions as Functions;

class NewsletterIntegration extends ServiceIntegration {

    /*
    |---------------------------------------------------------------------------------------------------
    | Constructor
    |---------------------------------------------------------------------------------------------------
    */
    public function __construct(){

    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Comprueba si la conexión con el servicio es exitosa
    |---------------------------------------------------------------------------------------------------
    */
    public function is_connect(){
        if( class_exists('Newsletter')){
            $this->service = \Newsletter::instance();
            return true;
        }
        $this->error = sprintf( __( 'Please install and activate the %s plugin.', 'masterpopups' ), '<a href="https://wordpress.org/plugins/newsletter/" target="_blank">Newsletter</a>' );
        return false;
    }


    /*
    |---------------------------------------------------------------------------------------------------
    | Retorna todas las listas
    |---------------------------------------------------------------------------------------------------
    */
    public function get_lists( $args = array() ){
        $items = $this->service->get_lists();
        if( empty( $items ) || ! is_array( $items ) ){
            return array();
        }
        foreach( $items as $list ){
            $items[$list->id] = $list->name;
        }
        return $items;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Verificar si el contacto está en la lista indicada
    |---------------------------------------------------------------------------------------------------
    */
    private function subscriber_exists( $email ){
        $email = strtolower( $email );
        $user = $this->service->get_user($email);
        if( ! $user ){
            return false;
        }
        return $user;
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Agrega un suscriptor a una lista
    |---------------------------------------------------------------------------------------------------
    */
    public function add_subscriber( $email, $data = array() ){
        $first_name = $data['first_name'];
        $first_name['value'] = ! empty( $first_name['value'] ) ? $first_name['value'] : '';
        $first_name['name'] = ! empty( $first_name['name'] ) ? $first_name['name'] : 'name';

        $last_name = $data['last_name'];
        $last_name['value'] = ! empty( $last_name['value'] ) ? $last_name['value'] : '';
        $last_name['name'] = ! empty( $last_name['name'] ) ? $last_name['name'] : 'surname';

        //Datos necesarios para la suscripción
        $params = array();
        $params['email'] = $email;
        $params['name'] = $first_name['value'];
        $params['surname'] = $last_name['value'];
        $params['custom_fields'] = array();

        //Importante. Rellena los campos del tipo {origin_url}, {ip}, {popup_title} con su valor real
        $params = $this->populate_render_fields( $params );

        //Comprobamos si el usuario ya está registrado
        $overwrite = isset( $data['overwrite'] ) && $data['overwrite'] == 'on' ? true : false;
        $subscriber = $this->subscriber_exists( $email );
        if( ! $overwrite && $subscriber ){
            $this->error = $this->messages['subscriber_exists'];
            return false;
        }
        if( !$subscriber ){
            $subscriber = new \TNP_User();
            $subscriber->status = \TNP_User::STATUS_CONFIRMED;
        }

        $subscriber->email = $params['email'];
        $subscriber->name = $params['name'];
        $subscriber->surname = $params['surname'];

        if( ! empty( $data['custom_fields'] ) ){
            $custom_fields = $this->get_custom_fields();
            foreach( $custom_fields as $cf_id => $cf_name ){
                if( isset( $data['custom_fields'][$cf_name] ) ){
                    $key = "profile_".$cf_id;
                    $subscriber->$key = $data['custom_fields'][$cf_name];
                }
            }
//            foreach( $data['custom_fields'] as $cf_name => $cf_value ){
//                $default_fields = $this->get_default_fields();
//                if( in_array( $cf_name, $default_fields ) ){
//                    $key = $data['custom_fields'][$cf_name];
//                    //$subscriber->$key = $cf_value;//Los valores que acepta son estrictos. da error en la BD
//                }
//            }
        }

        //Set to list
        $list_key = 'list_'.$this->list_id;
        $subscriber->$list_key = 1;
        //Suscripción a varias listas si existe más de una lista. Con $this->get_registered_lists(1) se itera a partir de la segunda lista.
        foreach( $this->get_registered_lists(1) as $list_id ){
            $list_key = 'list_'.$list_id;
            $subscriber->$list_key = 1;
        }
        $success = $this->service->save_user( $subscriber );
        if( !$success ){
            return false;
        }
        return true;
    }

    /*
	|---------------------------------------------------------------------------------------------------
	| Retorna todos los campos por defecto
	|---------------------------------------------------------------------------------------------------
	*/
    public function get_default_fields(){
        return array(
            'email',
            'name',
            'surname',
            //'sex',
            //'country',
            //'region',
            //'city',
        );
    }

    /*
    |---------------------------------------------------------------------------------------------------
    | Retorna todos los campos personalizados
    |---------------------------------------------------------------------------------------------------
    */
    public function get_custom_fields(){
        $items = array();
        $fields = $this->service->get_customfields();
        if( empty( $fields ) && !is_array( $fields ) ){
            return array();
        }
        foreach( $fields as $field ){
            $items[$field->id] = $field->name;
        }
        return $items;
    }

}
